//	NOTE: This file contains functions that are used in most of the other files


/* firefly constants*/
const FIREFLY_XUL_FILE = "chrome://firefly/content/firefly.xul";
const FIREFLY_APP_NAME = "Firefly";
const FIREFLY_LOG_FILE = 'firefly_log.html';


/*	NOTE: two functions to use for JSON
*/
function deserialize(serializedObject){
	try {
		var object = eval("(" + serializedObject + ")");
		return object;
	} catch (e) {
		firefly_Logger(e,'error');
	}
}
function serialize(object){
	try {
		return object.toSource();
	} catch (e) {
		firefly_Logger(e,'error');
	}
}


/*	NOTE: check if a value is present in an array
 *	array - array to search
 *	value - value to search
 *	return - index of the found value, false if the value is not found
*/
function firefly_inArray(array,value) {
	try {
		for (var i=0; i < array.length; i++) {
			// Matches identical (===), not just similar (==).
			if (array[i].toString() === value.toString()) {
				return i;
			}
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
    return false;
}


/*	NOTE: check if the search string exists in the array(or object) searches in subarrays too
 *	arr - array in which to search
 *	search - the search string
 *	return - true if the value exists, false if it doesn't
*/
function firefly_inObject(arr,search) {
	try {
		// alert(search);
		if (typeof(arr) == 'object') {
			for (var k in arr) {
				if (firefly_inObject(arr[k],search)) {
					return true;
				}
			}
		} else {
			if (arr.toString() === search.toString()) {
				return true;
			}
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return false;
}


/*	NOTE: erases all children of an object
 *	el - element whose childred to remove
 *	return - null
*/
function firefly_EraseContent(el,from) {
	try {
		if (from == undefined) {
			from = 0;
		}
		var cycles=el.childNodes.length;
		for (var p=from;p<cycles;p++) {
			el.removeChild(el.childNodes[from]);
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	NOTE: get the main window
 *	return - the current browser window with it's variables and functions
*/
function firefly_GetWindow() {
	try {
		var win = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
			   .getInterface(Components.interfaces.nsIWebNavigation)
			   .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
			   .rootTreeItem
			   .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
			   .getInterface(Components.interfaces.nsIDOMWindow);
		return win;
	} catch (e) {
		firefly_Logger(e,'error');
	}
}


/*	NOTE: get profile directory
 *	return - the profile directory file object
*/
function firefly_GetProfileDir() {
	try {
		return Components.classes["@mozilla.org/file/directory_service;1"]
				 .getService(Components.interfaces.nsIProperties)
				 .get("ProfD", Components.interfaces.nsIFile);
	} catch (e) {
		firefly_Logger(e,'error');
	}
}


/*	NOTE: get extension directory
 *	return fileobject of the extension directory
*/
function firefly_GetExtensionDir() {
	try {
		var GUID = "firefly.filemanager@gmail.com";
		var nsIExtensionManager = Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager);
		var extDir=nsIExtensionManager.getInstallLocation(GUID).getItemLocation(GUID);
		return extDir;
	} catch (e) {
		firefly_Logger(e,'error');
	}
}


/*	NOTE: get mozilla directory
 *	return fileobject of the mozilla directory
*/
function firefly_GetMozillaDir() {
	try {
		var file = Components.classes["@mozilla.org/file/directory_service;1"].
			getService(Components.interfaces.nsIProperties).
			get("CurProcD", Components.interfaces.nsIFile);
		return file;
	} catch (e) {
		firefly_Logger(e,'error');
	}
}


/*	NOTE: log errors and messages
 *	e - error object
 *	typeof - error,debug,log,...
 *
 *	every property of the object is shown
 *	if the property contains object or array show them too
 *
 *	return - null
*/
function firefly_Logger(e,type) {
	try {
		switch (type) {
			case 'log':
				var color = '#00b900';
				break;
			case 'debug':
				var color = '#0000ff';
				var debug = firefly_getPref('pref','bool','extensions.firefly.debug');
				if (!debug) {
					return null;
				}
				break;
			case 'error':
			default:
				var log_error = firefly_getPref('pref','bool','extensions.firefly.log_errors');
				if (!log_error) {
					return null;
				}
				var color = '#ff0000';
				break;
		}
		// NOTE: construct data
			var time=new Date();
			var data = '<div style="margin-top:10px;margin-bottom:10px;">';
			data += '<div><span style="color:'+color+';">Time:&nbsp;</span>'+(time.getDate() < 10 ? '0'+time.getDate():time.getDate())+'.'+((time.getMonth() + 1) < 10 ? '0'+(time.getMonth()+1):(time.getMonth() + 1))+'.'+
					time.getFullYear()+'\t'+time.getHours()+':'+(time.getMinutes() < 10?'0'+time.getMinutes():time.getMinutes())+':'+(time.getSeconds() < 10?'0'+time.getSeconds():time.getSeconds())+'</div>';
			for (var key in e) {
				if (typeof(e[key]) == 'object' || typeof(e[key]) == 'array') {
					data += '<div><span style="color:'+color+'">'+key+':</span>&nbsp;';
					for (var key1 in e[key]) {
						data += '<div>&nbsp;&nbsp;=>&nbsp;&nbsp;<span style="color:'+color+'">'+key1+':</span>&nbsp;'+e[key][key1]+'</div>';
					}
					data += '</div>';
				} else {
					data += '<div><span style="color:'+color+'">'+key+':</span>&nbsp;'+e[key]+'</div>';
				}
			}
			data += '</div>';

		// NOTE: saving to file
			var file = firefly_GetProfileDir();
			file.append(FIREFLY_LOG_FILE);
			var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
						 .createInstance(Components.interfaces.nsIFileOutputStream);

			if (file.exists()) {
				if (file.fileSize > 200000) {
					foStream.init(file,  0x02 | 0x08 | 0x20, 0664, 0); // write, create, truncate
				} else {
					foStream.init(file,  0x02 | 0x08 | 0x10, 0664, 0); // write, create, append
				}
			} else {
				foStream.init(file,  0x02 | 0x08 | 0x20, 0664, 0); // write, create, truncate
			}

			foStream.write(data, data.length);
			foStream.close();
		// NOTE: refreshing iframe
			// firefly_RefreshFrame();
	} catch(e) {
	}
	return null;
}


/*	Get preference from mozilla pref, sqlite, tab, other
 *	from - pref,tab,locale,...
 *	type - bool,char,int,comlpex|deserialize,''| properties file (example:"chrome://firefly/locale/start.properties")...
 *	name - name of the pref
 *
 *	return - the requested value
*/
function firefly_getPref(from,type,name) {
	try {
		switch (from) {
			case 'locale':
				var localeService = Components.classes["@mozilla.org/intl/nslocaleservice;1"].getService(Components.interfaces.nsILocaleService);
				var stringBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
				var stringBundle = stringBundleService.createBundle(type, localeService.getApplicationLocale());

				return stringBundle.GetStringFromName(name);
				break;
			case 'tab':
				var sessionStore = Components.classes["@mozilla.org/browser/sessionstore;1"].
								 getService(Components.interfaces.nsISessionStore);
				var currentTab = firefly_GetWindow().getBrowser().selectedTab;
				switch (type) {
					case 'deserialize':
						var value = sessionStore.getTabValue(currentTab, name);
						if (value != '') {
							return deserialize(value);
						} else {
							return '';
						}
						break;
					case '':
					default:
						return sessionStore.getTabValue(currentTab, name);
						break
				}
				break;
			case 'pref':
			default:
				var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
				switch (type) {
					case 'bool':
						return prefs.getBoolPref(name);
						break;
					case 'char':
						return prefs.getCharPref(name);
						break;
					case 'int':
						return prefs.getIntPref(name);
						break;
					case 'complex':
						try {
							return prefs.getComplexValue(name,Components.interfaces.nsISupportsString).data;
						} catch (e) {
							return null;
						}
						break;
				}
				break;
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	Set preference to mozilla pref, sqlite, tab, other
 *	to - pref,...
 *	type - bool,char,int,comlpex,...
 *	name - name of the pref
 *	value - value of the pref
 *
 *	return - null??
*/
function firefly_setPref(to,type,name,value) {
	try {
		switch (to) {
			case 'tab':
				var sessionStore = Components.classes["@mozilla.org/browser/sessionstore;1"].
								 getService(Components.interfaces.nsISessionStore);
				var currentTab = firefly_GetWindow().getBrowser().selectedTab;
				switch (type) {
					case 'serialize':
						return sessionStore.setTabValue(currentTab, name, serialize(value));
						break;
					case '':
					default:
						return sessionStore.setTabValue(currentTab, name, value);
						break;
				}
				break;
			case 'pref':
			default:
				var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
				switch (type) {
					case 'bool':
						return prefs.setBoolPref(name,value);
						break;
					case 'char':
						return prefs.setCharPref(name,value);
						break;
					case 'int':
						return prefs.setIntPref(name,value);
						break;
					case 'complex':
						var str = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);
						str.data = value;
						return prefs.setComplexValue(name,Components.interfaces.nsISupportsString,str);
						break;
				}
				break;
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	Delete preference from mozilla pref, sqlite, tab, other
 *	from - pref,tab,...
 *	name - name of the pref
 *
 *	return - null??
*/
function firefly_delPref(from,name) {
	try {
		switch (from) {
			case 'tab':
				var sessionStore = Components.classes["@mozilla.org/browser/sessionstore;1"].
							 getService(Components.interfaces.nsISessionStore);
				var currentTab = firefly_GetWindow().getBrowser().selectedTab;
				return sessionStore.deleteTabValue(currentTab, name);
				break;
			case 'pref':
			default:
				var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
				return prefs.deleteBranch(name);
				break;
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	NOTE: get currently active filebrowser
 *	return - currently active filebrowser object
*/
function firefly_GetCurrent(id) {
	try {
		if (id != undefined && id != 'firefly_sidebar_filebox') {
			return content.document.getElementById(id);
		} else if (id != 'firefly_sidebar_filebox') {
			id = firefly_getPref('tab','deserialize','currentFilebrowser');
		}
		if (id == 'firefly_sidebar_filebox') {
			// return document.getElementById("sidebar").contentDocument.getElementById(id);
			var sidebar = firefly_GetWindow().document.getElementById("sidebar");
			if (sidebar.contentWindow.location.href == "chrome://firefly/content/tree.xul") {
			  // call "yourNotificationFunction" in the sidebar's context:
			  return sidebar.contentDocument.getElementById(id);
			}
			// return content.document.getElementById(id);
		} else {
			if (content) {		// this is not working how to check if there is content???
				return content.document.getElementById(id);
			} else {
				return false;
			}
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return false;
}


/*	NOTE: Get the OS on which the extension runs
 *	return - the OS, if not possible to detect - return unknown
*/
function firefly_GetOS() {
	try {
		// var os = navigator.platform;
		var os = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULRuntime).OS;
		if (os.toLowerCase().indexOf('linux')!=-1) {
			return 'linux';
		}
		if (os.toLowerCase().indexOf('win')!=-1) {
			return 'windows';
		}
		if (os.toLowerCase().indexOf('mac')!=-1) {
			return 'mac';
		}
		if (os.toLowerCase().indexOf('power')!=-1) {
			return 'powerpc';
		}
		if (os.toLowerCase().indexOf('sun')!=-1) {
			return 'sunos';
		}
		if (os.toLowerCase().indexOf('sol')!=-1) {
			return 'solaris';
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return 'unknown';
}


/*	NOTE: Get the GET parameters
 *	name - the name of the parameter to get
 *	return - the value, empty if not present
*/
function firefly_GET(name) {
	try {
		name = name.replace(/[\[]/,"\\\[").replace(/[\]]/,"\\\]");
		var regexS = "[\\?&]"+name+"=([^&#]*)";
		var regex = new RegExp( regexS );
		var results = regex.exec( window.location.href );
		if( results == null ) {
			return "";
		} else {
			return results[1];
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
}


/*	NOTE: sets the view of the current filebox
 *	return - null
*/
function firefly_SetView(view,id) {
	try {
		firefly_GetCurrent(id).setAttribute('type',view);
		firefly_GetCurrent(id).refresh();
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	NOTE: opens the sidebar with the directory tree in it
 *	return - null
*/
function firefly_OpenSidebar() {
	try {
		firefly_GetWindow().toggleSidebar('firefly_Sidebar');
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	NOTE: creating the new tab that will host Firefly
 *	if current tab is blank use it
 *	if it is not blank - create new tab
 *	TODO: scan all tabs and if there is a blank one use it
 *	path - starting path
 *	openSame - true/false - whether to use the same tab
 *	return - null
*/
function firefly_OpenTab(path,openSame) {
	try {
		var pathValue = '';
		if (path != undefined) {
			pathValue = '?path=' + path.replace(/\\/g,'\\\\');
		}
		if (openSame != undefined && openSame == true) {
			openSame = true;				// open in the same tab no matter the current URL
		} else {
			openSame = false;
		}
		if (content.document.location != 'about:blank' && !openSame) {
			// var newTab = gBrowser.addTab(FIREFLY_XUL_FILE + pathValue);
			var newTab = firefly_GetWindow().getBrowser().addTab(FIREFLY_XUL_FILE + pathValue);
			newTab.setAttribute(FIREFLY_APP_NAME, 'true');
			// newTab.setAttribute('autoscroll','false');
			newTab.setAttribute('class','tabbrowser-tab');
			newTab.label = FIREFLY_APP_NAME;
			firefly_GetWindow().getBrowser().selectedTab = newTab;
		} else {
			firefly_GetWindow().getBrowser().selectedTab.setAttribute(FIREFLY_APP_NAME, 'true');
			firefly_GetWindow().getBrowser().selectedTab.setAttribute('class','tabbrowser-tab');
			firefly_GetWindow().getBrowser().selectedTab.label = FIREFLY_APP_NAME;
			// firefly_GetWindow().getBrowser().selectedBrowser.setAttribute('src',FIREFLY_XUL_FILE + pathValue);	// WEIRD: not working of the url is not about:blank next line is working
			setTimeout("firefly_GetWindow().getBrowser().selectedBrowser.setAttribute('src','"+FIREFLY_XUL_FILE + pathValue+"');",0);
		}
		// disable autoscroll because interferes with middle click action
		firefly_GetWindow().getBrowser().getBrowserForTab(firefly_GetWindow().getBrowser().selectedTab).setAttribute('autoscroll','false');
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	NOTE: reload iframe and scroll to bottom
 *	return - null
*/
function firefly_RefreshFrame() {
	try {
		var showLog = firefly_getPref('pref','bool','extensions.firefly.show_log');
		if (content.document.getElementById('firefly_LogWindow') && showLog) {
			content.document.getElementById('firefly_LogWindow').contentWindow.location.reload(true);
			setTimeout('content.document.getElementById("firefly_LogWindow").contentWindow.scrollTo(0,content.document.getElementById("firefly_LogWindow").contentWindow.document.childNodes[0].scrollHeight)',100);
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	NOTE: set the path of the view
 *	path - the path to set
 *	type - the type of view(details,list,....)
 *	provider - what the provider is (file for now later ftp,virtual, etc.)
 *	return - null
*/
function firefly_setPath(type,path,provider) {
	try {
		var current = firefly_GetCurrent();
		if (current == null) {
			return null;
		}
		// firefly_SetView(type);
		firefly_GetCurrent().setAttribute('type',type);
		current.setAttribute('path', path);
		current.path = path;
		current.setAttribute('provider', provider);
		// current.refresh();
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	NOTE: show help window
*/
function firefly_getHelp(sameWindow) {
	try {
		window.openDialog('chrome://firefly/content/options.xul',"firefly_options","chrome,titlebar,toolbar,centerscreen,modal,resizable=yes", 'help');
		/*if (sameWindow == undefined) {
			firefly_GetCurrent().setAttribute('path', 'help');
			firefly_SetView('help');
		} else {
			firefly_GetCurrent().setAttribute('path', 'help');
			firefly_SetView('help');
		}*/
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	NOTE: show start window
*/
function firefly_getStart(sameWindow) {
	try {
		if (sameWindow == undefined) {
			firefly_GetCurrent().setAttribute('path', 'start');
			firefly_SetView('start');
		} else {
			firefly_GetCurrent().setAttribute('path', 'start');
			firefly_SetView('start');
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	NOTE: Get the file object when given the path
 *	path - the path of the file
 *	return - the file object
*/
function firefly_GetFile(path) {
	try {
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance();
		if (file) file.QueryInterface(Components.interfaces.nsILocalFile);
		file.initWithPath(path);
		return file;
	} catch (e) {
		firefly_Logger(e,'error');
	}
}


/*	NOTE: load javascript file
 *	jsFile - path to the file to load
 *	return null
*/
function firefly_loadJs(jsFile) {
	try {
		var jsService = Components.classes["@mozilla.org/moz/jssubscript-loader;1"].getService(Components.interfaces.mozIJSSubScriptLoader);
		jsService.loadSubScript(jsFile,this);
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	NOTE: load provider of content (local files, ftp files...)
 *	provider - the provider to load
 *	return null
*/
function firefly_loadProvider(provider) {
	try {
		var jsFile;
		jsFile = 'chrome://firefly/content/js/providers/' + provider + '.js';
		firefly_loadJs(jsFile);
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


function firefly_changeAddressView(view,element) {
	try {
		if (view == 'breadcrumb') {
			if (document.getAnonymousNodes(element)) {
				var path = document.getAnonymousNodes(element)[0].childNodes[0].value;
				element.setAttribute('type', 'breadcrumb');
				element.setAttribute('path', path);
			}
		}
		if (view == 'text') {
			element.setAttribute('type', 'text');
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	NOTE: get configured tree/start page items
 *	return the deserialized object with items
*/
function firefly_getTree() {
	try {
		var tree = firefly_getPref('pref','complex','extensions.firefly.tree');
		tree = deserialize(tree);
		return tree;
	} catch (e) {
		firefly_Logger(e,'error');
	}
}


/*
 *	NOTE: show the dialog to choose the starting folders/drives
*/
function firefly_showStartSettings() {
	try {
		window.openDialog('chrome://firefly/content/options.xul',"","chrome,titlebar,toolbar,centerscreen,resizable=yes",'sidebar');
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	NOTE: Fire an event so that the properties dialog can understand something has changed and get the new values
 *
*/
function firefly_fireEvent(name, target) {
	try {
		//Ready: create a generic event
		var evt = document.createEvent("Events")
		//Aim: initialize it to be the event we want
		evt.initEvent(name, true, true); //true for can bubble, true for cancelable
		//FIRE!
		target.dispatchEvent(evt);
	} catch (e) {
		firefly_Logger(e,'error');
	}
}


/*	NOTE: call firefly_provider and get the correct provider for the type
 *	type - the requested provider type - default - 'file', in future - 'virtual','ftp'...
 *	return - the provider object
*/
function firefly_provider(type) {
	try {
		switch (type) {
			case 'file':
			default:
				return new firefly_file_provider(arguments[1], arguments[2], arguments[3]);
				break;
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return false;
}


/*	NOTE: builds keyboard shortcuts
 *
*/
function firefly_kbdShortcutsBuild() {
	try {
		// NOTE: char to code array
			var codesToKeys = new Object();
			codesToKeys['123'] = 'VK_F12';
			codesToKeys['122'] = 'VK_F11';
			codesToKeys['121'] = 'VK_F10';
			codesToKeys['120'] = 'VK_F9';
			codesToKeys['119'] = 'VK_F8';
			codesToKeys['118'] = 'VK_F7';
			codesToKeys['117'] = 'VK_F6';
			codesToKeys['116'] = 'VK_F5';
			codesToKeys['115'] = 'VK_F4';
			codesToKeys['114'] = 'VK_F3';
			codesToKeys['113'] = 'VK_F2';
			codesToKeys['112'] = 'VK_F1';
			codesToKeys['13'] = 'VK_RETURN';
			codesToKeys['27'] = 'VK_ESCAPE';
			codesToKeys['36'] = 'VK_HOME';
			codesToKeys['35'] = 'VK_END';
			codesToKeys['8'] = 'VK_BACK';
			codesToKeys['9'] = 'VK_TAB';
			codesToKeys['33'] = 'VK_PAGE_UP';
			codesToKeys['34'] = 'VK_PAGE_DOWN';
			codesToKeys['45'] = 'VK_INSERT';
			codesToKeys['46'] = 'VK_DELETE';
			codesToKeys['39'] = 'VK_RIGHT';
			codesToKeys['37'] = 'VK_LEFT';
			codesToKeys['38'] = 'VK_UP';
			codesToKeys['40'] = 'VK_DOWN';

		var mainKeyset = document.getElementById('mainKeyset');
		if (mainKeyset == undefined) {
			return;
		}
		var fireflyStart = 'firefly_';
		for (var i = 0;i< mainKeyset.childNodes.length;i++) {
			var kbdId = mainKeyset.childNodes[i].id;
			var startStr = kbdId.slice(0,fireflyStart.length);
			if (startStr == fireflyStart) {
				var key = mainKeyset.childNodes[i];
				var kbdShortcutName = kbdId.slice(fireflyStart.length, kbdId.length);
				var kbdPref = firefly_getPref('pref','complex','extensions.firefly.' + kbdShortcutName);
				kbdPref = deserialize(kbdPref);

				var modifiers = '';
				if (kbdPref['ctrlKey'] == true) {
					modifiers += 'accel ';
				}
				if (kbdPref['shiftKey'] == true) {
					modifiers += 'shift ';
				}
				if (kbdPref['altKey'] == true) {
					modifiers += 'alt ';
				}
				key.setAttribute('modifiers', modifiers);

				if (kbdPref['charCode'] != '') {
					key.setAttribute('key', String.fromCharCode(kbdPref['charCode']));
				} else if (kbdPref['keyCode'] != '') {
					key.setAttribute('keycode', codesToKeys[kbdPref['keyCode']]);
				}
			}
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
}


/*	NOTE:make transferable for clipboard or drag/drop
 *	action - what to do: cut/copy
 *	paths - array of file paths to go in the transferable
 *	return the transferable
*/
function firefly_makeTransferable(action, paths, source, sourceProvider) {
	try {
		// alert(paths);
		var transfer = new Object();
		transfer['provider'] = sourceProvider;
		transfer['source'] = source;
		transfer['paths'] = paths;
		transfer['action'] = action;
		// var copytext = serialize(paths);
		var copytext = serialize(transfer);
		var str = Components.classes["@mozilla.org/supports-string;1"]. createInstance(Components.interfaces.nsISupportsString);
		if (!str) return false;
		str.data = copytext;
		var trans = Components.classes["@mozilla.org/widget/transferable;1"]. createInstance(Components.interfaces.nsITransferable);
		if (!trans) return false;

		/*var flavor = '';
		switch (action) {
			case 'cut':
				flavor = 'application/firefly-cut-paths';
				break;
			case 'copy':
			default:
				flavor = 'application/firefly-copy-paths';
				break;
		}*/
		var flavor = 'application/firefly-paths';
		trans.addDataFlavor(flavor);
		trans.setTransferData(flavor,str,copytext.length * 2);
		return trans;
	} catch (e) {
		firefly_Logger(e,'error');
	}
}


/*	NOTE: using current XBL's method cut
 *	return - null
*/
function firefly_cut() {
	try {
		firefly_GetCurrent().cut();
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	NOTE: using current XBL's method copy
 *	return - null
*/
function firefly_cp() {
	try {
		firefly_GetCurrent().copy();
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	NOTE: using current XBL's method paste
 *	return - null
*/
function firefly_paste() {
	try {
		firefly_GetCurrent().paste();
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	NOTE: using current XBL's method delete
 *	return - null
*/
function firefly_delete() {
	try {
		firefly_GetCurrent().del();
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	NOTE: using current XBL's method selectSll
 *	return - null
*/
function firefly_selectAll() {
	try {
		firefly_GetCurrent().selectAll();
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	NOTE: using singlefile's rename function
 *
*/
function firefly_rename() {
	try {
		var selected = firefly_GetCurrent().getSelected();
		selected[0]['SingleFileBox'].rename();
	} catch (e) {
		firefly_Logger(e,'error');
	}
}


/*	NOTE: using current XBL's method createNew
 *	return - null
*/
function firefly_createNew(what, type) {
	try {
		firefly_GetCurrent().createNew(what, type);
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*
 *	NOTE: custom sort function to sort the matrixes when restoring a tab
 *	USED: firefly_HandleRestore
 *	return - 1|0|-1 depending on whether a>b|a=b|a<b
*/
function sortMatrixes(a,b) {
	try {
		if (a['matrix'] > b['matrix']) {
			return 1;
		}
		if (a['matrix'] < b['matrix']) {
			return -1;
		}
		if (a['matrix'] == b['matrix']) {
			return 0;
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*
 *	NOTE: this will run when a browser tab is restored
 *	return - null
*/
function firefly_HandleRestore(aEvent) {
	try {
		var fileboxes = firefly_getPref('tab','deserialize','fileboxes');
		if (fileboxes != '') {
			var mainBox = content.document.getElementById('mainBox');
			if (mainBox == null) {
				return;
			}
			mainBox.removeChild(mainBox.childNodes[0]);
			// var fileboxes = deserialize(fileboxes);
			// alert(fileboxes.toSource());
			var matrixes = new Array();
			var i = 0;
			for (key in fileboxes) {
				matrixes[i] = new Object();
				matrixes[i]['matrix'] = fileboxes[key]['matrix'];
				matrixes[i]['id'] = key;
				i++
			}
			// NOTE: sotring is needed to ensure the boxes are created in the correct order
			matrixes = matrixes.sort(sortMatrixes);
			var matrixesNew = new Object();
			for (key in matrixes) {
				matrixesNew[matrixes[key]['id']] = matrixes[key]['matrix'];
			}

			// for (key in fileboxes) {
			for (key in matrixesNew) {
				var tmpMainBox = mainBox;
				var matrixArr = fileboxes[key]['matrix'].split(',');		// split the matrix to array
				// alert(fileboxes[key].toSource());
				var last = matrixArr.splice(matrixArr.length - 1);			// remove the last element and save it in a variable
				for (var i = 0;i < matrixArr.length;i++) {
					if (matrixArr[i][0] == 1) {
						if (!tmpMainBox.childNodes[0]) {
							var newBox = document.createElement(matrixArr[i][1]+'box');
							newBox.flex = 1;
							tmpMainBox.appendChild(newBox);
						}
						tmpMainBox = tmpMainBox.childNodes[0]
					} else if (matrixArr[i][0] == 2) {
						if (!tmpMainBox.childNodes[1]) {
							tmpMainBox.appendChild(document.createElement('splitter'));
							var newBox = document.createElement(matrixArr[i][1]+'box');
							newBox.flex = 1;
							tmpMainBox.appendChild(newBox);
						}
						tmpMainBox = tmpMainBox.childNodes[2];
					} else {
						var e = new Object();
						e['message'] = 'Incorrect matrix for restoring';
						e['file'] = 'chrome://firefly/content/js/overlay.js';
						e['function'] = 'firefly_HandleRestore()';
						e['matrix'] = matrixArr[i].toSource();
						firefly_Logger(e,'error');
						return null;
					}
				}
				// creating the filebox
				/*var useAppend = true;
				if (tmpMainBox.childNodes[0] && tmpMainBox.childNodes[0].tagName == 'splitter') {			// NOTE: without this in the case - 1h,2h,2 ; 1h,2h,1 - will be restored as : splitter|filebox|filebox , not as filebox|splitter|filebox as it should
					useAppend = false;
				}*/
				if (last == 2) {
					tmpMainBox.appendChild(document.createElement('splitter'));
				}
				var newFilebox = document.createElement('firefly_filebox');
				newFilebox.id = key;
				newFilebox.flex = 1;
				newFilebox.setAttribute('path', fileboxes[key]['path']);	// TODO: uncomment
				newFilebox.setAttribute('type', fileboxes[key]['type']);	// TODO: uncomment
				newFilebox.setAttribute('provider', fileboxes[key]['provider']);	// TODO: uncomment
				tmpMainBox.appendChild(newFilebox);
				newFilebox.path = fileboxes[key]['path'];	// TODO: uncomment
				//newFilebox.refresh();
			}
			firefly_GetWindow().getBrowser().getBrowserForTab(firefly_GetWindow().getBrowser().selectedTab).setAttribute('autoscroll','false');
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}


/*	NOTE: create the element that will do the copy process
 *	action - what to do : cut/copy/delete
 *	paths - array with paths for the files being copied
 *	target -
 *	provider -
 *	source -
 *	sourceProvider -
*/
function firefly_doFiles(action, paths, target, provider, source, sourceProvider) {
	try {
		var newCopier = new firefly_copier(action, paths, provider, target, source, sourceProvider);
		var len = firefly_GetWindow().firefly_copy.push(newCopier);
		firefly_GetWindow().firefly_copy[len - 1].copyNum = len - 1;
		firefly_GetWindow().firefly_copy[len - 1].start();
	} catch (e) {
		firefly_Logger(e,'error');
	}
}


/*	NOTE: function that goes through all elements in an anonymous node and searches the correct internalid
 *	element - element whose anonymous content will be searched
 *	internalid - internalid to search
 *	return - the found element or undefined
 *	wish that function was available by default in the form element.getElementByInternalId(internalid)
*/
function firefly_getElementByInternalId(element, internalid) {
	try {
		var anonymousContent = document.getAnonymousNodes(element);
		if (anonymousContent) {
			for (var i = 0;i < anonymousContent.length;i++) {
				if (anonymousContent[i].getAttribute('internalid') == internalid) {
					return anonymousContent[i];
				}
				if (anonymousContent[i].childNodes.length > 0) {
					var el = firefly_checkNodesForInternalId(anonymousContent[i], internalid);
					if (el !== false) {
						return el;
					}
				}
			}
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return undefined;
}


/*	NOTE: check the nodes and subnodes for internalid attribute matching internalid
 *	element - the root element to start the recursion
 *	internalid - the value to search for
 *	return - the element where internalid was found or false
 *	this is helper function for firefly_getElementByInternalId
*/
function firefly_checkNodesForInternalId(element, internalid) {
	try {
		for (var i = 0;i < element.childNodes.length;i++) {
			if (element.childNodes[i].getAttribute('internalid') == internalid) {
				return element.childNodes[i];
			} else if (element.childNodes[i].childNodes.length > 0) {
				var found = firefly_checkNodesForInternalId(element.childNodes[i], internalid);
				if (found !== false) {
					return found;
				}
			}
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return false;
}


/*	NOTE: replace every occurence of %text% with the value of replaceArr[text]
 *	str - the string that needs replacement
 *	replaceArr - object that contains the key->value pairs that will be replaced
 *	return - the replaced string
*/
function firefly_replacePlaceholder(str, replaceArr) {
	try {
		for (var k in replaceArr) {
			var reg = new RegExp('%'+k+'%', 'g');
			str = str.replace(reg, replaceArr[k]);
		}
		return str;
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return '';
}


function firefly_formatSize(oldsize, type) {
	try {
		if (type == undefined) {
			type = 'size';
		}
		var size = 0;
		if (oldsize < 1024) {
			size = oldsize
			return size + ' B' + ((type == 'speed')?'/s':'');
		} else if (oldsize / 1024 < 1024) {
			size = oldsize / 1024;
			return Math.round(size * 100) / 100 + ' KB' + ((type == 'speed')?'/s':'');
		} else if (oldsize / (1024 * 1024) < 1024) {
			size = oldsize / (1024 * 1024);
			return  Math.round(size * 100) / 100 + ' MB' + ((type == 'speed')?'/s':'');
		} else if (oldsize / (1024 * 1024 * 1024) < 1024) {
			size = oldsize / (1024 * 1024 * 1024);
			return Math.round(size * 100) / 100 + ' GB' + ((type == 'speed')?'/s':'');
		} else if (oldsize / (1024 * 1024 * 1024 * 1024) < 1024) {
			size = oldsize / (1024 * 1024 * 1024 * 1024);
			return Math.round(size * 100) / 100 + ' TB' + ((type == 'speed')?'/s':'');
		}
		return 0 + ' B' + ((type == 'speed')?'/s':'');
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return '';
}


function firefly_getSettingsForType(file) {
	try {
		// get type
			var fileTypes = firefly_GetWindow().fileTypeSettings;
			var ext = file.getFileExt();
			var type = '';
			if (fileTypes[ext] == undefined) {
				type = '-other-';
			} else {
				type = '-' + fileTypes[ext]['type'] + '-';
			}
			/*if (fileTypes['type'] == undefined) {
				type = '-other-';
			} else {
				type = '-' + fileTypes['type'] + '-';
			}*/
			if (file.isDirectory()) {
				type = '-dir-';
			}

		// default program + arguments
			var defaultProgram = '';
			var defaultArguments = '';
			var relative = false;
			if (fileTypes[type]['defaultProgram'] != undefined && fileTypes[type]['defaultProgram'] != '') {
				defaultProgram = fileTypes[type]['defaultProgram'];
				defaultArguments = '';
				for (var k in fileTypes[type]['programs']) {
					if (fileTypes[type]['programs'][k]['path'] == defaultProgram) {
						defaultArguments = fileTypes[type]['programs'][k]['arguments'];
						if (fileTypes[type]['programs'][k]['relative'] == true) {
							var relative = true;
						}
					}
				}
				if (relative == true) {
					defaultProgram = firefly_GetMozillaDir().path + defaultProgram;
				}
			}

			if (fileTypes[ext] != undefined && fileTypes[ext]['defaultProgram'] != undefined && fileTypes[ext]['defaultProgram'] != '') {
				defaultProgram = fileTypes[ext]['defaultProgram'];
				defaultArguments = '';
				for (var k in fileTypes[ext]['programs']) {
					if (fileTypes[ext]['programs'][k]['path'] == defaultProgram) {
						defaultArguments = fileTypes[ext]['programs'][k]['arguments'];
						if (fileTypes[ext]['programs'][k]['relative'] == true) {
							var relative = true;
						}
					}
				}
				if (relative == true) {
					defaultProgram = firefly_GetMozillaDir().path + defaultProgram;
				}
			}
			if (defaultArguments == undefined) {
				defaultArguments = '';
			}

		// style
			var style = '';
			if (fileTypes[type]['style'] != null) {
				for (var k in fileTypes[type]['style']) {
					if (fileTypes[type]['style'][k] != null && fileTypes[type]['style'][k] != undefined && fileTypes[type]['style'][k] != '-') {
						style += k + ':' + fileTypes[type]['style'][k] + ';';
					}
				}
			}
			if (fileTypes[ext] != undefined && fileTypes[ext]['style'] != null) {
				for (var k in fileTypes[ext]['style']) {
					if (fileTypes[ext]['style'][k] != null && fileTypes[ext]['style'][k] != undefined && fileTypes[ext]['style'][k] != '-') {
						style += k + ':' + fileTypes[ext]['style'][k] + ';';
					}
				}
			}
			if (file.isDirectory()) {
				if (fileTypes['-dir-']['style'] != null) {
					for (var k in fileTypes['-dir-']['style']) {
						if (fileTypes['-dir-']['style'][k] != null && fileTypes['-dir-']['style'][k] != undefined && fileTypes['-dir-']['style'][k] != '-') {
							style += k + ':' + fileTypes['-dir-']['style'][k] + ';';
						}
					}
				}
			}

		// all programs and arguments
			var programs = new Array();
			if (fileTypes[type]['programs'] != undefined) {
				programs = fileTypes[type]['programs']
			}
			if (fileTypes[ext] != undefined && fileTypes[ext]['programs'] != undefined) {
				programs = fileTypes[ext]['programs']
			}

		// icons
			var icon16 = '';
			var icon32 = '';
			if (fileTypes[ext] != undefined && fileTypes[ext]['icons'] != undefined && fileTypes[ext]['icons']['16'] != undefined) {
				var icon16 = fileTypes[ext]['icons']['16'];
			}
			if (fileTypes[ext] != undefined && fileTypes[ext]['icons'] != undefined && fileTypes[ext]['icons']['32'] != undefined) {
				var icon32 = fileTypes[ext]['icons']['32'];
			}

		return [defaultProgram, defaultArguments, style, programs, icon16, icon32, type];
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return new Array();
}


function firefly_createPlayer() {
	try {
		var newPlayer = document.createElement('firefly_player');
		return newPlayer;
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}
function firefly_removePlayer(player) {
	try {
		if (player.parentNode) {
			player.parentNode.removeChild(player);
			return true;
		} else {
			return false;
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return false;
}
function firefly_pluginPresent(pluginName) {
	try {
		/* Check if the plugin exists - by the name displayed - not the file name - wouldn't be the same in Win/Linux (dll/so) */
		var plugins=navigator.plugins;
		for (var i = 0;i < plugins.length;i++) {
			if (plugins[i].filename.toLowerCase().indexOf(pluginName.toLowerCase()) != -1) {
				return true;
			}
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return false;
}

function firefly_formatTime(timeVal) {
	try {
		var timeHour = Math.round(timeVal / 1000);
		var timeSec = timeHour % 60;
		if( timeSec < 10 ) {
			timeSec = '0'+timeSec;
		}
		timeHour = (timeHour - timeSec)/60;
		var timeMin = timeHour % 60;
		if( timeMin < 10 ) {
			timeMin = '0'+timeMin;
		}
		timeHour = (timeHour - timeMin)/60;
		if( timeHour > 0 ) {
			return timeHour+":"+timeMin+":"+timeSec;
		} else {
			return timeMin+":"+timeSec;
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return '';
}
