//	NOTE: this file contains the functions used in the main Firefly window


/* show log window and load the log file */
function firefly_ShowLog() {
	try {
		if (document.getElementById("firefly_LogWindow")) {
			document.getElementById("firefly_LogWindow").style.display = '';
			document.getElementById("firefly_LogSplitter").style.display = '';
			var file = firefly_GetProfileDir();
			file.append(firefly_GetWindow().FIREFLY_LOG_FILE);

			document.getElementById("firefly_LogWindow").setAttribute('src','file://'+file.path);

			setTimeout('document.getElementById("firefly_LogWindow").contentWindow.scrollTo(0,document.getElementById("firefly_LogWindow").contentWindow.document.childNodes[0].scrollHeight)',100);
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
}


/*
 *	NOTE: check if the path is actually an alias
 *	alias - path to check
 *	return - true if it is alias, false otherwise
*/
function isAlias(alias) {
	try {
		if (alias == 'start') {
			return true;
		} else {
			var tree = firefly_getTree();
			for (var k in tree) {
				for (var t in tree[k]) {
					if (t == 'alias' && tree[k][t] == alias) {
						return true;
					}
				}
			}
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return false;
}


function getAlias(alias) {
	try {
		var tree = firefly_getTree();
		for (var k in tree) {
			// alert(tree[k]);
			for (var t in tree[k]) {
				if (t == 'alias' && tree[k][t] == alias) {
					return tree[k]['path'];
				}
				// alert(tree[k][t]);
			}
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return '';
}

function showAlias(alias,id) {
	try {
		if (alias == 'start') {
			firefly_SetView('start',id);
			return true;
		}

		var path = getAlias(alias);
		firefly_setPath(firefly_getPref('pref', 'char', 'extensions.firefly.defaultView'), path, 'file')
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return true;
}

function firefly_open() {
	try {
		var current = firefly_GetCurrent();
		var selected = current.getSelected();
		if (selected.length > 1) {
			return;  // if more than one item is selected... well we can't open few things at once - maybe in some other version
		} else {
			// alert(selected[0]);
			var evt = document.createEvent("MouseEvents");
			evt.initMouseEvent("dblclick", true, true, window, 2, 0, 0, 0, 0, false, false, false, false, 0, null);
			selected[0].dispatchEvent(evt);
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
}


function sortSubwindowsVert(el1,el2) {
	try {
		if (el1.boxObject.y<el2.boxObject.y) {
			return -1;
		} else if (el1.boxObject.y>el2.boxObject.y) {
			return 1;
		} else {
			return 0;
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
}
function sortSubwindowsHor(el1,el2) {
	try {
		if (el1.boxObject.x<el2.boxObject.x) {
			return -1;
		} else if (el1.boxObject.x>el2.boxObject.x) {
			return 1;
		} else {
			return 0;
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
}
function firefly_navigate(direction) {
	try {
		var fileboxes = document.getElementsByTagName('firefly_filebox');
		var element = firefly_GetCurrent();
		var x = element.boxObject.x;
		var y = element.boxObject.y;
		var views = new Array();
		for (var i = 0;i< fileboxes.length;i++) {
			views.push(fileboxes[i]);
		}
		var found = null;
		switch (direction) {
			case 'up':
				views.sort(sortSubwindowsVert);
				for (var i = 0;i< views.length;i++) {
					if (views[i].boxObject.y < y && views[i].boxObject.x == x) {
						found = views[i];
					} else if (found != null) {
						break;
					}
				}
				break;
			case 'down':
				views.sort(sortSubwindowsVert);
				for (var i = 0;i< views.length;i++) {
					if (views[i].boxObject.y > y && views[i].boxObject.x == x) {
						found = views[i];
						break;
					}
				}
				break;
			case 'left':
				views.sort(sortSubwindowsHor);
				for (var i = 0;i< views.length;i++) {
					if (views[i].boxObject.x < x && views[i].boxObject.y == y) {
						found = views[i];
					} else if (found != null) {
						break;
					}
				}
				break;
			case 'right':
				views.sort(sortSubwindowsHor);
				for (var i = 0;i< views.length;i++) {
					if (views[i].boxObject.x > x && views[i].boxObject.y == y) {
						found = views[i];
						break;
					}
				}
				break;
		}
		if (found && found!=null) {
			found.focused();
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return;
}

function contextSeparator(contextEl) {
	try {
		var newItem = document.createElement('menuseparator');
		contextEl.appendChild(newItem);
	} catch (e) {
		firefly_Logger(e,'error');
	}
}

function contextItem(contextEl, label, command, key, oncommand) {
	try {
		var newItem = document.createElement('menuitem');
		newItem.setAttribute('label', label);
		if (oncommand) {
			newItem.setAttribute('oncommand', command);
		} else {
			newItem.setAttribute('command', command);
		}
		newItem.setAttribute('key', key);
		contextEl.appendChild(newItem);
		return newItem;
	} catch (e) {
		firefly_Logger(e,'error');
	}
	return null;
}

function createContextMenu(contextType) {
	try {
		// var contextEl = document.getAnonymousNodes(el.parentNode.parentNode.parentNode)[1].childNodes[0];
		// var contextEl = firefly_getElementByInternalId(el.parentNode.parentNode.parentNode, 'rightClickPopup');
		// var contextEl = firefly_getElementByInternalId(el, 'rightClickPopup');
		var contextEl = document.getElementById('rightClickPopup');

		if (contextType == 'file') {
			// NOTE: execute
				if (!contextEl.files[0].isDirectory()) {
					[defaultProgram,defaultArguments,] = firefly_getSettingsForType(contextEl.files[0]);
					if (defaultProgram == '') {
						var execute = 'this.parentNode["files"][0].execute();';
						defaultProgram = '';
						arguments = '';
					} else {
						defaultProgram = defaultProgram;
						if (defaultArguments != '') {
							var arguments = defaultArguments.split(' ');
						} else {
							var arguments = new Array();
						}
						arguments.push(contextEl.files[0].path);
						arguments = arguments;
						var execute = 'var prog = firefly_GetWindow().firefly_provider(firefly_GetCurrent().getAttribute("provider"), this["defaultProgram"]);';
						execute += 'prog.launchWithArgs(this["arguments"]);';
					}

					var newItem = contextItem(contextEl, firefly_getPref('locale','chrome://firefly/locale/firefly.properties','open'), execute, 'firefly_kbdOpen', true);
					newItem.style.fontWeight = 'bold';
					newItem['defaultProgram'] = defaultProgram;
					newItem['arguments'] = arguments;
				}

			contextItem(contextEl, firefly_getPref('locale','chrome://firefly/locale/firefly.properties','rename'), 'firefly_cmdRename', 'firefly_kbdRename', false);
			contextSeparator(contextEl);
		}

		// NOTE: New... menu
			var newMenu = document.createElement('menu');
			newMenu.setAttribute('label',firefly_getPref('locale','chrome://firefly/locale/firefly.properties','NewMenu'));
			var newMenuPopup = document.createElement('menupopup');
			newMenu.appendChild(newMenuPopup);

			contextItem(newMenuPopup, firefly_getPref('locale','chrome://firefly/locale/firefly.properties','newFolder'), 'firefly_cmdNewFolder', 'firefly_kbdNewFolder', false);
			contextItem(newMenuPopup, firefly_getPref('locale','chrome://firefly/locale/firefly.properties','newFile'), 'firefly_cmdNewFile', 'firefly_kbdNewFile', false);
			// TODO: more new... items coming later
			contextEl.appendChild(newMenu);
		// NOTE: sorting
			var newMenu = document.createElement('menu');
			newMenu.setAttribute('label',firefly_getPref('locale','chrome://firefly/locale/firefly.properties','sortingOptions'));
			var newMenuPopup = document.createElement('menupopup');
			newMenu.appendChild(newMenuPopup);

			contextItem(newMenuPopup, firefly_getPref('locale','chrome://firefly/locale/firefly.properties','sortByName'), 'firefly_cmdSortName', 'firefly_kbdSortName', false);
			contextItem(newMenuPopup, firefly_getPref('locale','chrome://firefly/locale/firefly.properties','sortBySize'), 'firefly_cmdSortSize', 'firefly_kbdSortSize', false);
			contextItem(newMenuPopup, firefly_getPref('locale','chrome://firefly/locale/firefly.properties','sortByDate'), 'firefly_cmdSortDate', 'firefly_kbdSortDate', false);
			contextItem(newMenuPopup, firefly_getPref('locale','chrome://firefly/locale/firefly.properties','sortByExt'), 'firefly_cmdSortExtension', 'firefly_kbdSortExtension', false);
			// contextItem(newMenuPopup, firefly_getPref('locale','chrome://firefly/locale/firefly.properties','sortByNone'), 'firefly_cmdSortExtension', 'firefly_kbdSortExtension', false);

			contextEl.appendChild(newMenu);

		contextSeparator(contextEl);

		// NOTE: Open with menu
			var newMenu = document.createElement('menu');
			newMenu.setAttribute('label',firefly_getPref('locale','chrome://firefly/locale/firefly.properties','openWith'));
			var newMenuPopup = document.createElement('menupopup');
			newMenu.appendChild(newMenuPopup);

			var [defaultProgram,,,programs] = firefly_getSettingsForType(contextEl.files[0]);

			for (var k in programs) {
				var newItem = document.createElement('menuitem');

				if (programs[k]['relative'] == true) {
					var path = firefly_GetMozillaDir().path + programs[k]['path'];
				} else {
					var path = programs[k]['path'];
				}
				newItem['path'] = path;//programs[k]['path'];
				if (programs[k]['arguments'] != '') {
					var arguments = programs[k]['arguments'].split(' ');
				} else {
					var arguments = new Array();
				}
				arguments.push(contextEl.files[0].path);
				newItem['arguments'] = arguments;
				var execute = 'var prog = firefly_GetWindow().firefly_provider(firefly_GetCurrent().getAttribute("provider"), this["path"]);';
				execute += 'prog.launchWithArgs(this["arguments"]);';

				newItem.setAttribute('label',programs[k]['name']);
				if (path == defaultProgram) {
					newItem.style.fontWeight = 'bold';
				}
				newItem.setAttribute('oncommand',execute);
				newMenuPopup.appendChild(newItem);
			}

			var newItem = document.createElement('menuseparator');
			newMenuPopup.appendChild(newItem);

			var newItem = document.createElement('menuitem');
			newItem.setAttribute('label',firefly_getPref('locale','chrome://firefly/locale/firefly.properties','chooseProgram'));
			newItem.setAttribute('oncommand', 'firefly_GetWindow().openDialog("chrome://firefly/content/options.xul","","chrome,titlebar,toolbar,centerscreen,resizable=yes","fileTypes","'+contextEl.files[0].getFileExt()+'");');
			newMenuPopup.appendChild(newItem);

			contextEl.appendChild(newMenu);

		// NOTE: Send To menu
			var newMenu = document.createElement('menu');
			newMenu.setAttribute('label',firefly_getPref('locale','chrome://firefly/locale/firefly.properties','sendTo'));
			var newMenuPopup = document.createElement('menupopup');
			newMenu.appendChild(newMenuPopup);

			var sendToFolders = firefly_getPref('pref', 'complex', 'extensions.firefly.sendTo');
			sendToFolders = deserialize(sendToFolders);

			if (sendToFolders.length > 0) {
				for (var i = 0;i < sendToFolders.length;i++) {
					var newItem = document.createElement('menuitem');
					newItem['path'] = sendToFolders[i][0];//programs[k]['path'];
					newItem.setAttribute('label',sendToFolders[i][1]);
					var oncommand = '';
					if (contextType == 'general') {
						oncommand += 'firefly_selectAll();';
					}
					oncommand += 'firefly_sendTo("copy", this.path);';
					newItem.setAttribute('oncommand', oncommand);
					newMenuPopup.appendChild(newItem);
				}
			}

			var newItem = document.createElement('menuseparator');
			newMenuPopup.appendChild(newItem);

			var newItem = document.createElement('menuitem');
			newItem.setAttribute('label',firefly_getPref('locale','chrome://firefly/locale/firefly.properties','chooseProgram'));
			newItem.setAttribute('oncommand', 'firefly_GetWindow().openDialog("chrome://firefly/content/options.xul","","chrome,titlebar,toolbar,centerscreen,resizable=yes","sendTo");');
			newMenuPopup.appendChild(newItem);

			contextEl.appendChild(newMenu);

		// NOTE: Move To menu
			var newMenu = document.createElement('menu');
			newMenu.setAttribute('label',firefly_getPref('locale','chrome://firefly/locale/firefly.properties','moveTo'));
			var newMenuPopup = document.createElement('menupopup');
			newMenu.appendChild(newMenuPopup);

			var sendToFolders = firefly_getPref('pref', 'complex', 'extensions.firefly.sendTo');
			sendToFolders = deserialize(sendToFolders);

			if (sendToFolders.length > 0) {
				for (var i = 0;i< sendToFolders.length;i++) {
					var newItem = document.createElement('menuitem');
					newItem['path'] = sendToFolders[i][0];//programs[k]['path'];
					newItem.setAttribute('label',sendToFolders[i][1]);
					var oncommand = '';
					if (contextType == 'general') {
						oncommand += 'firefly_selectAll();';
					}
					oncommand += 'firefly_sendTo("cut", this.path);';
					newItem.setAttribute('oncommand', oncommand);
					newMenuPopup.appendChild(newItem);
				}
			}

			var newItem = document.createElement('menuseparator');
			newMenuPopup.appendChild(newItem);

			var newItem = document.createElement('menuitem');
			newItem.setAttribute('label',firefly_getPref('locale','chrome://firefly/locale/firefly.properties','chooseProgram'));
			newItem.setAttribute('oncommand', 'firefly_GetWindow().openDialog("chrome://firefly/content/options.xul","","chrome,titlebar,toolbar,centerscreen,resizable=yes","sendTo");');
			newMenuPopup.appendChild(newItem);

			contextEl.appendChild(newMenu);

		contextSeparator(contextEl);
		contextItem(contextEl, firefly_getPref('locale','chrome://firefly/locale/firefly.properties','cut'), 'firefly_cmdCut', 'firefly_kbdCut', false);
		contextItem(contextEl, firefly_getPref('locale','chrome://firefly/locale/firefly.properties','copy'), 'firefly_cmdCopy', 'firefly_kbdCopy', false);
		contextItem(contextEl, firefly_getPref('locale','chrome://firefly/locale/firefly.properties','paste'), 'firefly_cmdPaste', 'firefly_kbdPaste', false);
		contextItem(contextEl, firefly_getPref('locale','chrome://firefly/locale/firefly.properties','delete'), 'firefly_cmdDelete', 'firefly_kbdDelete', false);
		contextSeparator(contextEl);
		if (contextType == 'file') {
			// var fnCreateShortcuts = 'var files = this.parentNode["files"];for (var i = 0; i < files.length; i++){files[i].createShortcut();}';
			// contextItem(contextEl, firefly_getPref('locale','chrome://firefly/locale/firefly.properties','createShortcut'), fnCreateShortcuts, 'firefly_kbdMakeShortcut', true);
			contextItem(contextEl, firefly_getPref('locale','chrome://firefly/locale/firefly.properties','createShortcut'), 'firefly_cmdMakeShortcut', 'firefly_kbdMakeShortcut', false);
		}
		contextItem(contextEl, firefly_getPref('locale','chrome://firefly/locale/firefly.properties','selectAll'), 'firefly_cmdSelectAll', 'firefly_kbdSelectAll', false);
		contextSeparator(contextEl);
		// var props = 'window.openDialog("chrome://firefly/content/properties.xul","firefly_properties","chrome,titlebar,toolbar,centerscreen,resizable=yes", this["file"])';
		// var newItem = contextItem(contextEl, firefly_getPref('locale','chrome://firefly/locale/firefly.properties','properties'), props, 'firefly_kbdShowProperties', true);
		var newItem = contextItem(contextEl, firefly_getPref('locale','chrome://firefly/locale/firefly.properties','properties'), 'firefly_cmdShowProperties', 'firefly_kbdShowProperties', false);
		newItem['file'] = contextEl.files[0];
	} catch (e) {
		firefly_Logger(e,'error');
	}
}

function firefly_createShortcut() {
	try {
		var selected = firefly_GetCurrent().getSelected();
		for (var i = 0;i< selected.length;i++) {
			selected[i].SingleFileBox.file.createShortcut();
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
}
function firefly_showProperties() {
	try {
		if (firefly_GetCurrent().getSelected().length > 0) {
			var file = firefly_GetCurrent().getSelected()[0].SingleFileBox.file;
		} else {
			var hidden = firefly_getPref('pref','bool','extensions.firefly.hidden_files');
			var file = firefly_GetWindow().firefly_provider(firefly_GetCurrent().getAttribute('provider'), firefly_GetCurrent().path, hidden);
		}
		window.openDialog("chrome://firefly/content/properties.xul","firefly_properties","chrome,titlebar,toolbar,centerscreen,resizable=yes", file);
	} catch (e) {
		firefly_Logger(e,'error');
	}
}


/*	NOTE: do sendto moveto from the context menu
 *	action - what to do - copy/cut
 *	toWhere - target folder
*/
function firefly_sendTo(action, toWhere) {
	try {
		var selected = firefly_GetCurrent().getSelected();
		var paths = new Array();
		for (var k in selected) {
			// alert(selected[k]['SingleFileBox'].file.file);
			paths.push(selected[k]['SingleFileBox'].file.path);
		}
		// TODO: when more providers are available in the send to array will have to be entered the provider too and here will be used that provider
		firefly_doFiles(action, paths, toWhere, firefly_GetCurrent().getAttribute('provider'), firefly_GetCurrent().path, firefly_GetCurrent().getAttribute('provider'));
	} catch (e) {
		firefly_Logger(e,'error');
	}
}

// SORTING FUNCTIONS
	function sortByField(field) {
		try {
			var sortDir = firefly_GetCurrent().sortDir;
			if (sortDir == '') {
				sortDir = 'asc';
			} else if (sortDir == 'asc') {
				sortDir = 'desc';
			} else {
				sortDir = 'asc';
			}
			firefly_GetCurrent().sortBy = field;
			firefly_GetCurrent().sortDir = sortDir;
			firefly_GetCurrent().buildFromFiles();
		} catch (e) {
			firefly_Logger(e,'error');
		}
	}

	function sort(sortBy, sortDir, sortArr) {
		try {
			var sorted = new Array();

			switch (sortBy) {
				case 'name':
					if (sortDir == 'asc') {
						sortArr.sort(nameSortAsc);
					} else {
						sortArr.sort(nameSortDesc);
					}
					break;
				case 'size':
					if (sortDir == 'asc') {
						sortArr.sort(sizeSortAsc);
					} else {
						sortArr.sort(sizeSortDesc);
					}
					break;
				case 'date':
					if (sortDir == 'asc') {
						sortArr.sort(dateSortAsc);
					} else {
						sortArr.sort(dateSortDesc);
					}
					break;
				case 'ext':
					if (sortDir == 'asc') {
						sortArr.sort(extSortAsc);
					} else {
						sortArr.sort(extSortDesc);
					}
					break;
			}

			return sortArr;
		} catch (e) {
			firefly_Logger(e,'error');
		}
	}

	function nameSortAsc(a, b) {
		try {
			if (a.leafName.toLowerCase() > b.leafName.toLowerCase()) {
				return 1;
			} else if (a.leafName.toLowerCase() == b.leafName.toLowerCase()) {
				return 0;
			} else {
				return -1;
			}
		} catch (e) {
			firefly_Logger(e,'error');
		}
	}
	function nameSortDesc(a, b) {
		if (a.leafName.toLowerCase() > b.leafName.toLowerCase()) {
			return -1;
		} else if (a.leafName.toLowerCase() == b.leafName.toLowerCase()) {
			return 0;
		} else {
			return 1;
		}
	}

	function sizeSortAsc(a, b) {
		try {
			if (a.fileSize > b.fileSize) {
				return 1;
			} else if (a.fileSize == b.fileSize) {
				return 0;
			} else {
				return -1;
			}
		} catch (e) {
			firefly_Logger(e,'error');
		}
	}
	function sizeSortDesc(a, b) {
		try {
			if (a.fileSize > b.fileSize) {
				return -1;
			} else if (a.fileSize == b.fileSize) {
				return 0;
			} else {
				return 1;
			}
		} catch (e) {
			firefly_Logger(e,'error');
		}
	}

	function dateSortAsc(a, b) {
		try {
			if (a.lastModifiedTime > b.lastModifiedTime) {
				return 1;
			} else if (a.lastModifiedTime == b.lastModifiedTime) {
				return 0;
			} else {
				return -1;
			}
		} catch (e) {
			firefly_Logger(e,'error');
		}
	}
	function dateSortDesc(a, b) {
		try {
			if (a.lastModifiedTime > b.lastModifiedTime) {
				return -1;
			} else if (a.lastModifiedTime == b.lastModifiedTime) {
				return 0;
			} else {
				return 1;
			}
		} catch (e) {
			firefly_Logger(e,'error');
		}
	}

	function extSortAsc(a, b) {
		try {
			if (a.getFileExt() > b.getFileExt()) {
				return 1;
			} else if (a.getFileExt() == b.getFileExt()) {
				return 0;
			} else {
				return -1;
			}
		} catch (e) {
			firefly_Logger(e,'error');
		}
	}
	function extSortDesc(a, b) {
		try {
			if (a.getFileExt() > b.getFileExt()) {
				return -1;
			} else if (a.getFileExt() == b.getFileExt()) {
				return 0;
			} else {
				return 1;
			}
		} catch (e) {
			firefly_Logger(e,'error');
		}
	}
// END SORTING FUNCTIONS

var doingLinked = false;
function firefly_doLinked(newPath) {
	try {
		if (doingLinked) {
			return;
		}
		// var currentId = firefly_GetCurrent();
		var linkedArr = firefly_getPref('tab', 'deserialize', 'linked');
		if (linkedArr == '') {
			linkedArr = new Array();
		}

		if (linkedArr.length == 0) {
			return;
		}
		var linkedObj = firefly_getPref('tab','deserialize','linkedPaths');
		if (linkedObj == null || linkedObj == '') {
			linkedObj = new Object();
		}

		var currentId = firefly_getPref('tab', 'deserialize', 'currentFilebrowser');
		if (newPath.indexOf(linkedObj[currentId]) == -1) {
			return;		// new string is not part of the base path => we are not interested
		}
		var subpath = newPath.substr(linkedObj[currentId].length);

		doingLinked = true;
		if (firefly_inArray(linkedArr,currentId) !== false) {
			for (var i = 0;i< linkedArr.length;i++) {
				if (linkedArr[i].toString() != currentId.toString()) {
					var path = linkedObj[linkedArr[i]] + subpath;
					firefly_GetCurrent(linkedArr[i]).path = path;
					firefly_GetCurrent(linkedArr[i]).refresh();
				}
			}
		}
		doingLinked = false;
	} catch (e) {
		firefly_Logger(e,'error');
	}
}

function firefly_showTooltip(event, file) {
	try {
		var showTooltipPreviews = firefly_getPref('pref','bool','extensions.firefly.showTooltipPreviews');
		var tooltipEl = document.getElementById('tooltipPopup');

		[,,style,,,icon,type] = firefly_getSettingsForType(file);
		if (icon == '') {
			var icon = file.getIcon(32);
		}
		var imgBox = tooltipEl.childNodes[0];

		if ((type == '-picture-' || type == '-movie-' || type == '-audio-') && firefly_GetCurrent().getAttribute('type') != 'gallery' && showTooltipPreviews) {
			if (globalPlayer != null) {
				firefly_removePlayer(globalPlayer);
			}
			firefly_EraseContent(imgBox);
			var newPlayer = firefly_createPlayer();
			newPlayer.setAttribute('controls', 'false');
			newPlayer.setAttribute('path', file.path);
			newPlayer.setAttribute('provider', firefly_GetCurrent().getAttribute('provider'));
			newPlayer.style.width = '300px';
			newPlayer.style.height = '300px';

			imgBox.appendChild(newPlayer);
			globalPlayer = newPlayer;
		} else {
			firefly_EraseContent(imgBox);
			var img = document.createElement('image');
			img.setAttribute('src', icon);
			img.setAttribute('name', 'fileIcon');
			img.setAttribute('align', 'center');
			img.setAttribute('pack', 'center');
			img.setAttribute('class', 'tootltipFileIcon');
			imgBox.appendChild(img);
		}
		var descBoxes = tooltipEl.childNodes[1];
		for (var i = 0; i < descBoxes.childNodes.length;i++) {
			if (descBoxes.childNodes[i].getAttribute('name') == 'fileName') {
				descBoxes.childNodes[i].value = file.leafName; // file name

				descBoxes.childNodes[i].setAttribute('style',style);
			}
			if (descBoxes.childNodes[i].getAttribute('name') == 'filePath') {
				descBoxes.childNodes[i].value = file.path; // file path
			}
			if (descBoxes.childNodes[i].getAttribute('name') == 'filePermissions') {
				descBoxes.childNodes[i].value = file.getPermissions(); // file permissions
			}
			if (descBoxes.childNodes[i].getAttribute('name') == 'fileSize') {
				descBoxes.childNodes[i].value = file.getFileSize(); // file size
			}
			if (descBoxes.childNodes[i].getAttribute('name') == 'fileDate') {
				descBoxes.childNodes[i].value = file.getFileDate(); // file date
			}
		}
		tooltipEl.showPopup(event.target,event.screenX,event.target.boxObject.screenY , "tooltip");
	} catch (e) {
		firefly_Logger(e,'error');
	}
}
function firefly_hideTooltip() {
	try {
		var tooltipEl = document.getElementById('tooltipPopup');
		if (globalPlayer != null && firefly_GetCurrent() && firefly_GetCurrent().getAttribute('type') != 'gallery') {
			firefly_removePlayer(globalPlayer);
		}
		var imgBox = tooltipEl.childNodes[0];
		firefly_EraseContent(imgBox);
		tooltipEl.hidePopup();
	} catch (e) {
		firefly_Logger(e,'error');
	}
}
function firefly_doMiddleClick(event, file) {
	try {
		if (file.isDirectory()) {
			if (event.target.parentNode.parentNode.parentNode.parentNode.id == 'firefly_sidebar_filebox') {
				return;
			}
			var action = firefly_getPref('pref', 'char', 'extensions.firefly.middleClickOnFolder');
			switch (action) {
				case 'open':
					this.dblclick(event);
					break;
				case 'newTab':
					firefly_OpenTab(file.path);
					break;
				case 'SplitV':
					firefly_GetWindow().firefly_Split('v', file.path);
					break;
				case 'splitH':
					firefly_GetWindow().firefly_Split('h', file.path);
					break;
				case 'ask':
				default:
					//var contextEl = document.getAnonymousNodes(this.parentNode.parentNode.parentNode)[1].childNodes[2];
					//var contextEl = firefly_getElementByInternalId(this.parentNode.parentNode.parentNode, 'middleClickPopup');
					var contextEl = document.getElementById('middleClickPopup');
					contextEl['file'] = file;
					contextEl.showPopup(event.target,event.screenX,event.target.boxObject.screenY, "context");
					break;
			}
		}
	} catch (e) {
		firefly_Logger(e,'error');
	}
}

function firefly_showContextMenu(event) {
	try {
		var contextEl = document.getElementById('rightClickPopup');
		firefly_EraseContent(contextEl);
		if (event.target.SingleFileBox) {
			var contextType = 'file';
			var file = event.target.SingleFileBox.file;
			var target = event.target;
		} else if(event.target.parentNode.SingleFileBox) {
			var contextType = 'file';
			var file = event.target.parentNode.SingleFileBox.file;
			var target = event.target.parentNode;
		} else {
			var contextType = 'general';
			var hidden = firefly_getPref('pref','bool','extensions.firefly.hidden_files');
			var file = firefly_GetWindow().firefly_provider(firefly_GetCurrent().getAttribute('provider'), firefly_GetCurrent().path, hidden);
			var target = firefly_GetCurrent();
		}

		var fileArr = new Array();
		fileArr.push(file);
		contextEl['files'] = fileArr;
		contextEl['fileTypes'] = firefly_GetWindow().fileTypeSettings;

		createContextMenu(contextType);

		//contextEl.showPopup(event.target, event.clientX, event.clientY, "context");+
		contextEl.showPopup(event.target, event.screenX,event.target.boxObject.screenY, "context");
		// contextEl.showPopup(target,-1,-1 , "context", "bottomright", "bottomright");
	} catch (e) {
		firefly_Logger(e,'error');
	}
}

function firefly_showFilter() {
	try {
		var popup = firefly_GetWindow().document.getElementById('firefly_PredefinedFilters');
		popup.showPopup(popup.parentNode, -1, -1, "popup");
		popup.focus();
		popup.childNodes[0].childNodes[0].focus();
	} catch (e) {
		firefly_Logger(e,'error');
	}
}
