// global vars for thread management
	var firefly_activeThreads = [];
	var firefly_waitingThreads = [];
	var firefly_threadCount = 0;
	// var firefly_threadPriority = 10;
	var firefly_threadPriority = firefly_getPref('pref', 'int', 'extensions.firefly.folderScanPriority');


/*
 *	NOTE: starting the initial thread manager
*/
function firefly_spawn(generator) {
	return new Thread(generator).start();
}


/*
 *	NOTE: Putting the thread to sleep
 *	millis - how much time to wait
 *	return - null
*/
function firefly_sleep(millis) {
	setTimeout((yield CONTINUATION), millis);
	yield SUSPEND;
}


/*
 *	NOTE: Get the file object when given the path
 *	path - the path of the file
 *	return - the file object
*/
function firefly_GetFile(path) {
	var file = Components.classes["@mozilla.org/file/local;1"].createInstance();
	if (file) file.QueryInterface(Components.interfaces.nsILocalFile);
	file.initWithPath(path);
	return file;
}


/*
 *	NOTE: Get all files and folder on the specified path
 *	startPath - the path of the folder to start with
 *	recursive - should we take all subfolders and their files too
 *	return - dirs - collected dirs  and  files - collected files
*/
function firefly_getFiles(startPath,recursive) {
	yield firefly_sleep(firefly_threadPriority);
	var files = new Array();
	var dirs = new Array();
	var folder = firefly_GetFile(startPath);
	dirs.push(folder);
	var i = 0;
	while (folder = dirs[i]) {
		var file = null;
		var folderEntries=folder.directoryEntries;
		while (folderEntries.hasMoreElements()) {
			file=folderEntries.getNext().QueryInterface(Components.interfaces.nsILocalFile);
			if (file.isDirectory()) {
				dirs.push(file);
			} else {
				files.push(file);
			}
			yield firefly_sleep(firefly_threadPriority);
		}
		if (!recursive) {
			break;
		}
		i++;
		yield firefly_sleep(firefly_threadPriority);
	}
	firefly_GetCurrent().dirs = dirs;
	firefly_GetCurrent().files = files;
	firefly_GetCurrent().rebuild();
}


/*
 *	NOTE: A thread manager that will take care of the threads
*/
var firefly_spawnManager = function () {
	firefly_spawn(
		function () {
			// this thread runs for the lifetime of the page
			while (true) {
				// wait for all running threads to end
				while (firefly_activeThreads.length) {
					yield firefly_activeThreads.shift().join();
				}
				// launch a thread if one is waiting
				if (firefly_waitingThreads.length) {
					var thread = firefly_waitingThreads.shift();
					firefly_activeThreads.push(thread);
					thread.start();
				}
				// wait for new threads if necessary
				while (!firefly_activeThreads.length && !firefly_waitingThreads.length) {
					yield firefly_sleep(firefly_threadPriority);
				}
			}
		}
	)
};
window.addEventListener('load',firefly_spawnManager ,false);


/*
 *	NOTE: Make a new thread for getting folder contents
 *	path - starting path
 *	recursive - if it is recursive folder scan
 *	return - the thread
*/
/*function firefly_createThread(path,recursive) {
	firefly_threadCount++;
	return new Thread(firefly_getFiles(path,recursive));
}*/
function firefly_createThread(spawnFunction) {
	firefly_threadCount++;
	return new Thread(spawnFunction);
}

/*
 *	NOTE: Start a thread immediately
 *	path - starting path
 *	recursive - if it is recursive folder scan
 *	return - null
*/
function firefly_startImmediate(spawnFunction) {
	var thread = firefly_createThread(spawnFunction);
	firefly_activeThreads.push(thread);
	thread.start();
	return null;
}


/*
 *	NOTE: Start a thread when there are no active threads
 *	path - starting path
 *	recursive - if it is recursive folder scan
 *	return - null
*/
function firefly_startOnIdle(spawnFunction) {
	firefly_waitingThreads.push(firefly_createThread(spawnFunction));
	return null;
}
